package org.openstreetmap.josm

import org.eclipse.jgit.storage.file.FileRepositoryBuilder
import org.eclipse.jgit.revwalk.filter.MessageRevFilter
import org.eclipse.jgit.revwalk.RevWalk
import org.eclipse.jgit.lib.Repository
import org.eclipse.jgit.api.Git
import org.eclipse.jgit.revwalk.RevCommit

import org.gradle.api.DefaultTask
import org.gradle.api.tasks.TaskAction
import org.gradle.api.tasks.OutputFile

import java.text.SimpleDateFormat
import java.util.regex.Pattern;

class RevisionTask extends DefaultTask {
    @OutputFile
    File revisionFile;
    
    File repositoryDirectory;

    private String svnRevision = null;
    def revName, revDate;
    
    def loadRevision() {
        loadSvnRevision();
        
        if (svnRevision == null) {
            loadGitRevision();
        }
    }
    
    def loadSvnRevision() {
        def env = System.getenv().collect { k, v -> "$k=$v" }
        env << "LANG=C";
        def infoXML = "svn info --xml .".execute(env, repositoryDirectory).text
        def info = new XmlSlurper().parseText(infoXML);
        svnRevision = info.entry.commit.@revision
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSSS'Z'");
        revDate = format.parse(info.entry.commit.date.text());
    }
    
    def loadGitRevision() {
        FileRepositoryBuilder builder = new FileRepositoryBuilder();
        Repository repository = builder.readEnvironment().findGitDir(repositoryDirectory).build();

        RevWalk walk = new RevWalk(repository);
        RevCommit headCommit = walk.parseCommit(repository.resolve("HEAD"));
        walk.markStart(headCommit);
        def pattern = '.*git-svn-id: [^@]*@([0-9]+).*([\\w-]+)\\s*$'
        walk.setRevFilter(MessageRevFilter.create(pattern))

        def svnImportRevision = walk.iterator().next();

        svnRevision = '?'
        if (svnImportRevision) {
            def m = Pattern.compile(pattern).matcher(svnImportRevision.fullMessage);
            m.find();
            svnRevision = m.group(1);
        }
        repository.close();

        revName = headCommit.id.name
        //NPE: revDate = headCommit.committerIdent.time;
        revDate = new Date()
    }
    
    def getSvnRevision() {
        if (!svnRevision) {
            loadRevision();
        }
        return svnRevision;
    }

    @TaskAction
    def createCommitFile() {
        revisionFile.parentFile.mkdirs()
        revisionFile.text = sprintf("""
# automatically generated by JOSM build file - do not edit
Revision: %s
Is-Local-Build: true
Build-Date: %s
GIT-Revision: %s (%s)
""", getSvnRevision(), (new Date()).format('yyyy-MM-dd-HH-mm-ss'), revName, revDate.format('yyyy-MM-dd-HH-mm-ss'))
    }
    /*onlyIf {
        !commitFile.isFile() || !commitFile.text.contains(ext.revName)
    }*/
}